@@ -1,7 +1,7 @@
 package CPAN::SQLite::CPANMeta;
 
-our $DATE = '2015-01-08'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2015-01-09'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -13,6 +13,8 @@ use Archive::Tar;
 use File::Slurp::Tiny qw(read_file);
 #use File::Temp qw(tempdir);
 use JSON;
+use Module::CoreList;
+use Version::Util qw(version_gt);
 use YAML::Syck ();
 
 use Exporter;
@@ -351,6 +353,60 @@ ORDER BY dist_file
 
 # XXX cache connection?
 
+sub _get_prereqs {
+    my ($mod, $dbh, $memory, $level, $max_level, $phase, $rel, $include_core, $plver) = @_;
+
+    $log->tracef("Finding dependencies for module %s (level=%i) ...", $mod, $level);
+
+    # first find out which distribution that module belongs to
+    my $sth = $dbh->prepare("SELECT dist_id FROM mods WHERE mod_name=?");
+    $sth->execute($mod);
+    my $modrec = $sth->fetchrow_hashref;
+    return [404, "No such module: $mod"] unless $modrec;
+
+    # fetch the dependency information
+    $sth = $dbh->prepare("SELECT
+  CASE WHEN dp.mod_id THEN (SELECT mod_name FROM mods WHERE mod_id=dp.mod_id) ELSE dp.mod_name END AS module,
+  phase,
+  rel,
+  version
+FROM deps dp
+WHERE dp.dist_id=?
+ORDER BY module");
+    $sth->execute($modrec->{dist_id});
+    my @res;
+    while (my $row = $sth->fetchrow_hashref) {
+        next unless $phase eq 'ALL' || $row->{phase} eq $phase;
+        next unless $rel   eq 'ALL' || $row->{rel}   eq $rel;
+        #say "include_core=$include_core, is_core($row->{module}, $row->{version}, $plver)=", Module::CoreList::is_core($row->{module}, $row->{version}, version->parse($plver)->numify);
+        next if !$include_core && Module::CoreList::is_core($row->{module}, $row->{version}, version->parse($plver)->numify);
+        if (defined $memory->{$row->{module}}) {
+            if (version_gt($row->{version}, $memory->{$row->{module}})) {
+                $memory->{$row->{version}} = $row->{version};
+            }
+            next;
+        }
+        delete $row->{phase} unless $phase eq 'ALL';
+        delete $row->{rel}   unless $rel   eq 'ALL';
+        $row->{level} = $level;
+        push @res, $row;
+        $memory->{$row->{module}} = $row->{version};
+    }
+
+    if (@res && ($max_level==-1 || $level < $max_level)) {
+        my $i = @res-1;
+        while ($i >= 0) {
+            my $subres = _get_prereqs($res[$i]{module}, $dbh, $memory,
+                                      $level+1, $max_level, $phase, $rel, $include_core, $plver);
+            $i--;
+            next if $subres->[0] != 200;
+            splice @res, $i+2, 0, @{$subres->[2]};
+        }
+    }
+
+    [200, "OK", \@res];
+}
+
 $SPEC{'deps_cpan_meta'} = {
     v => 1.1,
     summary => 'Query dependency from CPAN::SQLite database',
@@ -373,6 +429,31 @@ $SPEC{'deps_cpan_meta'} = {
             }],
             default => 'requires',
         },
+        level => {
+            summary => 'Recurse for a number of levels (-1 means unlimited)',
+            schema  => 'int*',
+            default => 1,
+            cmdline_aliases => {
+                l => {},
+                R => {
+                    summary => 'Recurse (alias for `--level -1`)',
+                    is_flag => 1,
+                    code => sub { $_[0]{level} = -1 },
+                },
+            },
+        },
+        include_core => {
+            summary => 'Include Perl core modules',
+            'summary.alt.bool.not' => 'Exclude Perl core modules',
+            schema  => 'bool',
+            default => 0,
+        },
+        perl_version => {
+            summary => 'Set base Perl version for determining core modules',
+            schema  => 'str*',
+            default => "$^V",
+            cmdline_aliases => {V=>{}},
+        },
     },
 };
 sub deps_cpan_meta {
@@ -386,36 +467,24 @@ sub deps_cpan_meta {
     my $mod     = $args{module};
     my $phase   = $args{phase} // 'runtime';
     my $rel     = $args{rel} // 'requires';
+    my $plver   = $args{perl_version} // "$^V";
+    my $level   = $args{level} // 1;
+    my $include_core = $args{include_core} // 0;
 
     my $db_path = "$db_dir/$db_name";
     $log->tracef("Connecting to SQLite database at %s ...", $db_path);
     my $dbh = DBI->connect("dbi:SQLite:dbname=$db_path", undef, undef,
                            {RaiseError=>1});
 
-    # first find out which distribution that module belongs to
-    my $sth = $dbh->prepare("SELECT dist_id FROM mods WHERE mod_name=?");
-    $sth->execute($mod);
-    my $modrec = $sth->fetchrow_hashref;
-    return [404, "No such module: $mod"] unless $modrec;
+    my $res = _get_prereqs($mod, $dbh, {}, 1, $level, $phase, $rel, $include_core, $plver);
 
-    my @res;
-
-    # fetch the dependency information
-    $sth = $dbh->prepare("SELECT
-  CASE WHEN dp.mod_id THEN (SELECT mod_name FROM mods WHERE mod_id=dp.mod_id) ELSE dp.mod_name END AS module,
-  phase,
-  rel,
-  version
-FROM deps dp
-WHERE dp.dist_id=?
-ORDER BY module");
-    $sth->execute($modrec->{dist_id});
-    while (my $row = $sth->fetchrow_hashref) {
-        next unless $rel   eq 'ALL' || $row->{rel}   eq $rel;
-        next unless $phase eq 'ALL' || $row->{phase} eq $phase;
-        push @res, $row;
+    return $res unless $res->[0] == 200;
+    for (@{$res->[2]}) {
+        $_->{module} = ("  " x ($_->{level}-1)) . $_->{module};
+        delete $_->{level};
     }
-    [200, "OK", \@res];
+
+    $res;
 }
 
 $SPEC{'revdeps_cpan_meta'} = {
@@ -463,7 +532,7 @@ CPAN::SQLite::CPANMeta - Index and query CPAN Meta information in CPAN::SQLite d
 
 =head1 VERSION
 
-This document describes version 0.02 of CPAN::SQLite::CPANMeta (from Perl distribution CPAN-SQLite-CPANMeta), released on 2015-01-08.
+This document describes version 0.03 of CPAN::SQLite::CPANMeta (from Perl distribution CPAN-SQLite-CPANMeta), released on 2015-01-09.
 
 =head1 SYNOPSIS
 
@@ -491,7 +560,7 @@ extracted/recorded are: dependencies (tables: C<files>, C<deps>).
 
 =head2 deps_cpan_meta(%args) -> [status, msg, result, meta]
 
-Query dependency from CPAN::SQLite database.
+{en_US Query dependency from CPAN::SQLite database}.
 
 Arguments ('*' denotes required arguments):
 
@@ -499,18 +568,30 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-Location of your local CPAN mirror, e.g. /path/to/cpan.
+{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
 
 =item * B<db_dir> => I<str>
 
-Database directory, defaults to your CPAN home.
+{en_US Database directory, defaults to your CPAN home}.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-Database name.
+{en_US Database name}.
+
+=item * B<include_core> => I<bool> (default: 0)
+
+{en_US Include Perl core modules}.
+
+=item * B<level> => I<int> (default: 1)
+
+{en_US Recurse for a number of levels (-1 means unlimited)}.
 
 =item * B<module>* => I<str>
 
+=item * B<perl_version> => I<str> (default: "v5.18.4")
+
+{en_US Set base Perl version for determining core modules}.
+
 =item * B<phase> => I<str> (default: "runtime")
 
 =item * B<rel> => I<str> (default: "requires")
@@ -531,7 +612,7 @@ Return value:  (any)
 
 =head2 index_cpan_meta(%args) -> [status, msg, result, meta]
 
-Index CPAN Meta information into CPAN::SQLite database.
+{en_US Index CPAN Meta information into CPAN::SQLite database}.
 
 Arguments ('*' denotes required arguments):
 
@@ -539,15 +620,15 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-Location of your local CPAN mirror, e.g. /path/to/cpan.
+{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
 
 =item * B<db_dir> => I<str>
 
-Database directory, defaults to your CPAN home.
+{en_US Database directory, defaults to your CPAN home}.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-Database name.
+{en_US Database name}.
 
 =back
 
@@ -565,7 +646,7 @@ Return value:  (any)
 
 =head2 revdeps_cpan_meta(%args) -> [status, msg, result, meta]
 
-Query reverse dependencies from CPAN::SQLite database.
+{en_US Query reverse dependencies from CPAN::SQLite database}.
 
 Arguments ('*' denotes required arguments):
 
@@ -573,15 +654,15 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-Location of your local CPAN mirror, e.g. /path/to/cpan.
+{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
 
 =item * B<db_dir> => I<str>
 
-Database directory, defaults to your CPAN home.
+{en_US Database directory, defaults to your CPAN home}.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-Database name.
+{en_US Database name}.
 
 =item * B<module>* => I<str>
 
@@ -609,9 +690,11 @@ information" and is a SQLite database that contains more information about CPAN
 from various sources, e.g.: number of ratings from
 L<http://cpanratings.perl.org>, upload date and test results from CPAN Testers,
 number of tickets from L<http://rt.cpan.org>. However, generating CPANDB locally
-will require downloading gigabytes of data and take hours. An online cache is
-available and can be used via L<CPANDB> but you cannot control the update
-frequency.
+will require downloading gigabytes of data and take hours (generating
+CPAN::SQLite database + CPAN::SQLite::CPANMeta, on the other hand, only takes a
+few minutes; regular updating typically only takes a few seconds). An online
+cache is available and can be used via L<CPANDB> but you cannot control the
+update frequency.
 
 =head1 HOMEPAGE
 
